unit mix_engine;


interface

uses
  MMSystem;

  procedure SetVol(linetype: integer; percentage: integer);
  procedure SetVolumeMute(linetype:Integer; muted:Boolean);
  function GetVol(linetype: integer): integer;
  function GetVolumeMute(linetype:Integer): Boolean;
implementation


function GetVolumeControl(aMixer: HMixer; componentType, ctrlType:Longint; var mxc: TMixerControl): Boolean;
var
  mxl: TMixerLine;
  mxlc: TMixerLineControls; 
  rc: Longint; 
begin 
  Result := FALSE; 
  FillChar(mxl, SizeOf(TMixerLine), 0); 
  mxl.cbStruct := SizeOf(TMixerLine);
  mxl.dwComponentType := componentType;
  // Obtain a line corresponding to the component type
  rc := mixerGetLineInfo(aMixer, @mxl, MIXER_GETLINEINFOF_COMPONENTTYPE);
  if rc = MMSYSERR_NOERROR then
  begin
    mxlc.cbStruct := SizeOf(TMixerLineControls);
    mxlc.dwLineID := mxl.dwLineID; 
    mxlc.dwControlType := ctrlType; 
    mxlc.cControls := 1; 
    mxlc.cbmxctrl := SizeOf(TMixerLine); 
    mxlc.pamxctrl := @mxc;
    mxlc.pamxctrl^.cbStruct := SizeOf(TMixerControl);
    mixerGetLineControls(aMixer, @mxlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
    rc := mixerGetLineControls(aMixer, @mxlc, MIXER_GETLINECONTROLSF_ONEBYTYPE);
    Result := rc = MMSYSERR_NOERROR; 
  end; 
end;


function SetVolumeControl(aMixer: HMixer; mxc: TMixerControl; volume:LongInt): Boolean;
var 
  mxcd: TMixerControlDetails; 
  vol: TMixerControlDetails_Unsigned; 
  rc: MMRESULT; 
begin 
  FillChar(mxcd, SizeOf(mxcd), 0); 
  mxcd.dwControlID := mxc.dwControlID; 
  mxcd.cbStruct := SizeOf(TMixerControlDetails); 
  mxcd.cbDetails := SizeOf(TMixerControlDetails_Unsigned); 
  mxcd.paDetails := @vol; 
  mxcd.cChannels := 1; 
  vol.dwValue := volume; 
  rc := mixerSetControlDetails(aMixer, @mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
  Result := rc = MMSYSERR_NOERROR; 
end;


function InitMixer: HMixer;
var
  Err: MMRESULT;
begin
  Err := mixerOpen(@Result, 0, 0, 0, 0);
  if Err <> MMSYSERR_NOERROR then Result := 0;
end;



//////////////////////////////////////////////////////////////////
//   procedure SetVol                                           //
//                                                              //
//   This is the procedure that is called from the plugin       //
//   to set the volume                                          //
//                                                              //
//   accepts a volume from 0 to 100                             //
//////////////////////////////////////////////////////////////////
procedure SetVol(linetype: integer; percentage: integer);
var
  MyMixerHandle: HMixer;
  MyVolCtrl: TMixerControl;
  vol: integer;
begin
  vol := (65535 * percentage) div 100;
  MyMixerHandle := InitMixer;
  if MyMixerHandle <> 0 then
  try
    FillChar(MyVolCtrl, SizeOf(MyVolCtrl), 0);
    if GetVolumeControl(
      MyMixerHandle,
      linetype,
      MIXERCONTROL_CONTROLTYPE_VOLUME,
      MyVolCtrl) then
    begin
      SetVolumeControl(MyMixerHandle, MyVolCtrl, vol);
    end;
  finally
    mixerClose(MyMixerHandle);
  end;
end;

//////////////////////////////////////////////////////////////////
//   function GetVol                                            //
//                                                              //
//   Returns the current volume as a percentage from 0 to 100   //
//////////////////////////////////////////////////////////////////
function GetVol(linetype: integer): integer;
var
  MyMixerHandle: HMixer;
  MyVolCtrl: TMixerControl;
  mxcd: TMixerControlDetails;
  vol: TMixerControlDetails_Unsigned;
begin
  MyMixerHandle := InitMixer;
  result := 0;
  if MyMixerHandle <> 0 then
  try
    FillChar(MyVolCtrl, SizeOf(MyVolCtrl), 0);
    if GetVolumeControl(
      MyMixerHandle,
      linetype,
      MIXERCONTROL_CONTROLTYPE_VOLUME,
      MyVolCtrl) then
    begin
        FillChar(mxcd, SizeOf(mxcd), 0);
        mxcd.dwControlID := MyVolCtrl.dwControlID;
        mxcd.cbStruct := SizeOf(TMixerControlDetails);
        mxcd.cbDetails := SizeOf(TMixerControlDetails_Unsigned);
        mxcd.paDetails := @vol;
        mxcd.cChannels := 1;
        mixerGetControlDetails(MyMixerHandle, @mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
        result := (vol.dwValue * 100) div 65535;
    end;
  finally
    mixerClose(MyMixerHandle);
  end;
end;




function GetVolumeMute(linetype:Integer): Boolean;
var
  MyMixerHandle: HMixer;
  mxc: MIXERCONTROL;
  mcdMute: MIXERCONTROLDETAILS_BOOLEAN;
  mxcd: TMIXERCONTROLDETAILS;
  intRet: integer;
begin
  MyMixerHandle := InitMixer;
  if MyMixerHandle <> 0 then
  try
    if GetVolumeControl(
      MyMixerHandle,
      linetype,
      MIXERCONTROL_CONTROLTYPE_MUTE,
      mxc) then
    begin
       FillChar(mxcd, SizeOf(mxcd),0);
       mxcd.cbStruct := SizeOf(TMIXERCONTROLDETAILS);
       mxcd.dwControlID := mxc.dwControlID;
       mxcd.cChannels := 1;
       mxcd.cbDetails := SizeOf(MIXERCONTROLDETAILS_BOOLEAN);
       mxcd.paDetails := @mcdMute;

       // Get mute
      intRet := mixerGetControlDetails(MyMixerHandle, @mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
       if mcdMute.fValue = 0 then Result:=false
       else Result := True;
    end;

  finally
    mixerClose(MyMixerHandle);
  end;

end;


procedure SetVolumeMute(linetype:Integer; muted:Boolean);
var
  MyMixerHandle: HMixer;
  mxc: MIXERCONTROL;
  mcdMute: MIXERCONTROLDETAILS_BOOLEAN;
 mxcd: TMIXERCONTROLDETAILS;
begin
  MyMixerHandle := InitMixer;
  if MyMixerHandle <> 0 then
  try
    if GetVolumeControl(
      MyMixerHandle,
      linetype,
      MIXERCONTROL_CONTROLTYPE_MUTE,
      mxc) then
    begin
       FillChar(mxcd, SizeOf(mxcd),0);
       mxcd.cbStruct := SizeOf(TMIXERCONTROLDETAILS);
       mxcd.dwControlID := mxc.dwControlID;
       mxcd.cChannels := 1;
       mxcd.cbDetails := SizeOf(MIXERCONTROLDETAILS_BOOLEAN);
       mxcd.paDetails := @mcdMute;

       // Set and UnSet  mute
       mcdMute.fValue := Ord(muted);
       mixerSetControlDetails(MyMixerHandle, @mxcd, MIXER_SETCONTROLDETAILSF_VALUE);
    end;

  finally
    mixerClose(MyMixerHandle);
  end;

end;

end.
